@@ -35,19 +35,31 @@
 * messages
 ************************************************************************/
 
+#include "config.h"
+
 #include <assert.h>
 #include <stdarg.h>
+#ifndef UPNP_USE_BCBPP
+#ifndef UPNP_USE_MSVCPP
+ #include <stdint.h>
+#endif
+#endif
 
-#include <arpa/inet.h>
-#include <netinet/in.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/time.h>
-#include <sys/wait.h>
-#include <unistd.h>
-#include <sys/utsname.h>
+#ifndef WIN32
+ #include <arpa/inet.h>
+ #include <netinet/in.h>
+ #include <sys/types.h>
+ #include <sys/socket.h>
+ #include <sys/time.h>
+ #include <sys/wait.h>
+ #include <unistd.h>
+ #include <sys/utsname.h>
+ #include <fcntl.h>
+#else
+ #include <winsock2.h>
+ #include <malloc.h>
+#endif
 #include "unixutil.h"
-#include "config.h"
 #include "upnp.h"
 #include "upnpapi.h"
 #include "membuffer.h"
@@ -155,6 +167,12 @@ http_Connect( IN uri_type * destination_url,
 
     if( connect( connfd, ( struct sockaddr * )&url->hostport.IPv4address,
                  sizeof( struct sockaddr_in ) ) == -1 ) {
+#ifdef WIN32
+		DBGONLY(
+			UpnpPrintf(UPNP_CRITICAL, HTTP, __FILE__, __LINE__,
+			"connect error: %d\n", WSAGetLastError());
+		)
+#endif
         shutdown( connfd, SD_BOTH );
         UpnpCloseSocket( connfd );
         return UPNP_E_SOCKET_CONNECT;
@@ -217,7 +235,7 @@ http_RecvMessage( IN SOCKINFO * info,
                     if( parser->content_length >
                         ( unsigned int )g_maxContentLength ) {
                     *http_error_code = HTTP_REQ_ENTITY_TOO_LARGE;
-                    return UPNP_E_BAD_HTTPMSG;
+                    return UPNP_E_OUTOF_BOUNDS;
                 }
 
                 return 0;
@@ -293,8 +311,8 @@ http_SendMessage( IN SOCKINFO * info,
     char *filename = NULL;
     FILE *Fp;
     int num_read,
-      num_written,
-      amount_to_be_read = 0;
+      num_written;
+    off_t amount_to_be_read = 0;
     va_list argp;
     char *file_buf = NULL,
      *ChunkBuf = NULL;
@@ -354,7 +372,7 @@ http_SendMessage( IN SOCKINFO * info,
                     return UPNP_E_FILE_READ_ERROR;
                 }
             } else if( Instr && Instr->IsRangeActive ) {
-                if( fseek( Fp, Instr->RangeOffset, SEEK_CUR ) != 0 ) {
+                if( fseeko( Fp, Instr->RangeOffset, SEEK_CUR ) != 0 ) {
                     free( ChunkBuf );
                     return UPNP_E_FILE_READ_ERROR;
                 }
@@ -624,13 +642,14 @@ http_Download( IN const char *url_str,
     *temp = '/';
     DBGONLY( UpnpPrintf
              ( UPNP_INFO, HTTP, __FILE__, __LINE__,
-               "HOSTNAME : %s Length : %d\n", hoststr, hostlen );
+               "HOSTNAME : %s Length : %zu\n", hoststr, hostlen );
          )
 
-        ret_code = http_MakeMessage( &request, 1, 1, "QsbcDCUc",
-                                     HTTPMETHOD_GET, url.pathquery.buff,
-                                     url.pathquery.size, "HOST: ", hoststr,
-                                     hostlen );
+        ret_code = http_MakeMessage(
+            &request, 1, 1,
+            "QsbcDCUc",
+            HTTPMETHOD_GET, url.pathquery.buff, url.pathquery.size,
+            "HOST: ", hoststr, hostlen );
     if( ret_code != 0 ) {
         DBGONLY( UpnpPrintf
                  ( UPNP_INFO, HTTP, __FILE__, __LINE__,
@@ -694,7 +713,7 @@ http_Download( IN const char *url_str,
 
         // save mem for body only
         *document = realloc( msg_start, *doc_length + 1 );  //LEAK_FIX_MK
-        //*document = Realloc( msg_start,msg_length, *doc_length + 1 );//LEAK_FIX_MK
+        // *document = Realloc( msg_start,msg_length, *doc_length + 1 );//LEAK_FIX_MK
 
         // shrink can't fail
         assert( ( int )msg_length > *doc_length );
@@ -783,21 +802,27 @@ MakePostMessage( const char *url_str,
          )
 
         if( contentLength >= 0 ) {
-        ret_code = http_MakeMessage( request, 1, 1, "QsbcDCUTNc",
-                                     HTTPMETHOD_POST, url->pathquery.buff,
-                                     url->pathquery.size, "HOST: ",
-                                     hoststr, hostlen, contentType,
-                                     contentLength );
+        ret_code = http_MakeMessage(
+            request, 1, 1,
+            "QsbcDCUTNc",
+            HTTPMETHOD_POST, url->pathquery.buff, url->pathquery.size,
+            "HOST: ", hoststr, hostlen,
+            contentType,
+            (off_t)contentLength );
     } else if( contentLength == UPNP_USING_CHUNKED ) {
-        ret_code = http_MakeMessage( request, 1, 1, "QsbcDCUTKc",
-                                     HTTPMETHOD_POST, url->pathquery.buff,
-                                     url->pathquery.size, "HOST: ",
-                                     hoststr, hostlen, contentType );
+        ret_code = http_MakeMessage(
+            request, 1, 1,
+            "QsbcDCUTKc",
+            HTTPMETHOD_POST, url->pathquery.buff, url->pathquery.size,
+            "HOST: ", hoststr, hostlen,
+            contentType );
     } else if( contentLength == UPNP_UNTIL_CLOSE ) {
-        ret_code = http_MakeMessage( request, 1, 1, "QsbcDCUTc",
-                                     HTTPMETHOD_POST, url->pathquery.buff,
-                                     url->pathquery.size, "HOST: ",
-                                     hoststr, hostlen, contentType );
+        ret_code = http_MakeMessage(
+            request, 1, 1,
+            "QsbcDCUTc",
+            HTTPMETHOD_POST, url->pathquery.buff, url->pathquery.size,
+            "HOST: ", hoststr, hostlen,
+            contentType );
     } else {
         ret_code = UPNP_E_INVALID_PARAM;
     }
@@ -852,7 +877,7 @@ http_WriteHttpPost( IN void *Handle,
 
     if( ( !handle ) || ( !size ) || ( ( ( *size ) > 0 ) && !buf )
         || ( ( *size ) < 0 ) ) {
-        ( *size ) = 0;
+        if(size) ( *size ) = 0;
         return UPNP_E_INVALID_PARAM;
     }
     if( handle->contentLength == UPNP_USING_CHUNKED ) {
@@ -862,6 +887,9 @@ http_WriteHttpPost( IN void *Handle,
             tempbuf =
                 ( char * )malloc( ( *size ) + CHUNK_HEADER_SIZE +
                                   CHUNK_TAIL_SIZE );
+
+            if ( tempbuf == NULL) return UPNP_E_OUTOF_MEMORY;
+
             sprintf( tempbuf, "%x\r\n", ( *size ) );    //begin chunk
             tempSize = strlen( tempbuf );
             memcpy( tempbuf + tempSize, buf, ( *size ) );
@@ -1041,6 +1069,7 @@ typedef struct HTTPGETHANDLE {
     http_parser_t response;
     SOCKINFO sock_info;
     int entity_offset;
+    int cancel;
 } http_get_handle_t;
 
 /************************************************************************
@@ -1048,6 +1077,7 @@ typedef struct HTTPGETHANDLE {
 *																		
 * Parameters:															
 *	const char *url_str ;	String as a URL
+*	const char *proxy_str ;	String as a URL of proxy to use
 *	membuffer *request ;	Buffer containing the request									
 *	uri_type *url ; 		URI object containing the scheme, path 
 *							query token, etc.
@@ -1061,11 +1091,14 @@ typedef struct HTTPGETHANDLE {
 ************************************************************************/
 int
 MakeGetMessage( const char *url_str,
+                const char *proxy_str,
                 membuffer * request,
                 uri_type * url )
 {
     int ret_code;
     char *urlPath = alloca( strlen( url_str ) + 1 );
+    int querylen = 0;
+    const char *querystr;
     int hostlen = 0;
     char *hoststr,
      *temp;
@@ -1102,10 +1135,19 @@ MakeGetMessage( const char *url_str,
                          "HOSTNAME : %s Length : %d\n", hoststr, hostlen );
          )
 
-        ret_code = http_MakeMessage( request, 1, 1, "QsbcDCUc",
-                                     HTTPMETHOD_GET, url->pathquery.buff,
-                                     url->pathquery.size, "HOST: ",
-                                     hoststr, hostlen );
+    if( proxy_str ) {
+        querystr = url_str;
+        querylen = strlen( querystr );
+    } else {
+        querystr = url->pathquery.buff;
+        querylen = url->pathquery.size;
+    }
+
+    ret_code = http_MakeMessage(
+        request, 1, 1,
+        "QsbcDCUc",
+        HTTPMETHOD_GET, querystr, querylen,
+        "HOST: ", hoststr, hostlen );
 
     if( ret_code != 0 ) {
         DBGONLY( UpnpPrintf( UPNP_INFO, HTTP, __FILE__, __LINE__,
@@ -1254,7 +1296,7 @@ ReadResponseLineAndHeaders( IN SOCKINFO * info,
 *	Parameters :
 *		IN void *Handle :			Handle to the HTTP get object
 *		IN OUT char *buf :			Buffer to get the read and parsed data
-*		IN OUT unsigned int *size :	Size of tge buffer passed
+*		IN OUT unsigned int *size :	Size of the buffer passed
 *		IN int timeout :			time out value
 *
 *	Description :	Parses already existing data, then gets new data.
@@ -1265,6 +1307,7 @@ ReadResponseLineAndHeaders( IN SOCKINFO * info,
 *		UPNP_E_INVALID_PARAM  - Invalid Parameter;
 *		UPNP_E_BAD_RESPONSE ;
 *		UPNP_E_BAD_HTTPMSG ;
+*		UPNP_E_CANCELED ;
 *
 *	Note :
 ************************************************************************/
@@ -1285,7 +1328,7 @@ http_ReadHttpGet( IN void *Handle,
 
     if( ( !handle ) || ( !size ) || ( ( ( *size ) > 0 ) && !buf )
         || ( ( *size ) < 0 ) ) {
-        ( *size ) = 0;
+        if(size) ( *size ) = 0;
         return UPNP_E_INVALID_PARAM;
     }
     //first parse what has already been gotten
@@ -1308,6 +1351,7 @@ http_ReadHttpGet( IN void *Handle,
     //read more if necessary entity
     while( ( ( handle->entity_offset + ( *size ) ) >
              handle->response.msg.entity.length )
+           && ( ! handle->cancel )
            && ( handle->response.position != POS_COMPLETE ) ) {
         num_read =
             sock_read( &handle->sock_info, tempbuf, sizeof( tempbuf ),
@@ -1366,9 +1410,73 @@ http_ReadHttpGet( IN void *Handle,
                                           handle->entity_offset],
             ( *size ) );
     handle->entity_offset += ( *size );
+
+    if ( handle->cancel )
+        return UPNP_E_CANCELED;
+
     return UPNP_E_SUCCESS;
 }
 
+/************************************************************************
+*	Function :	http_HttpGetProgress
+*
+*	Parameters :
+*		IN void *Handle :			Handle to the HTTP get object
+*		OUT unsigned int *length :	Buffer to get the read and parsed data
+*		OUT unsigned int *total :	Size of tge buffer passed
+*
+*	Description :	Extracts information from the Handle to the HTTP get
+*					object.
+*
+*	Return : int ;
+*		UPNP_E_SUCCESS - On Sucess ;
+*		UPNP_E_INVALID_PARAM  - Invalid Parameter;
+*
+*	Note :
+************************************************************************/
+int http_HttpGetProgress( IN void *Handle, 
+                      OUT unsigned int *length,
+                      OUT unsigned int *total )
+{
+    http_get_handle_t *handle = Handle;
+
+    if( ( !handle ) || ( !length ) || ( !total ) ) {
+        return UPNP_E_INVALID_PARAM;
+    }
+    *length = handle->response.msg.entity.length;
+    *total = handle->response.content_length;
+    return UPNP_E_SUCCESS;
+}
+
+/************************************************************************
+*	Function :	http_CancelHttpGet
+*
+*	Parameters :
+*		IN void *Handle ;	Handle to HTTP get object
+*
+*	Description :	Set the cancel flag of the HttpGet handle
+*
+*	Return : int ;
+*		UPNP_E_SUCCESS - On Success
+*		UPNP_E_INVALID_PARAM - Invalid Parameter
+*
+*	Note :
+************************************************************************/
+int
+http_CancelHttpGet( IN void *Handle )
+{
+    http_get_handle_t *handle = Handle;
+
+    if( !handle ) {
+        return UPNP_E_INVALID_PARAM;
+    }
+
+    handle->cancel = 1;
+
+    return UPNP_E_SUCCESS;
+}
+
+
 /************************************************************************
 *	Function :	http_CloseHttpGet
 *
@@ -1434,6 +1542,47 @@ http_OpenHttpGet( IN const char *url_str,
                   OUT int *contentLength,
                   OUT int *httpStatus,
                   IN int timeout )
+{
+    return http_OpenHttpGetProxy(url_str, NULL, Handle, contentType, contentLength, httpStatus, timeout);
+}
+
+/************************************************************************
+*	Function :	http_OpenHttpGetProxy
+*
+*	Parameters :
+*		IN const char *url_str :	String as a URL
+*		IN const char *proxy_str :	String as a URL
+*		IN OUT void **Handle :		Pointer to buffer to store HTTP
+*									post handle
+*		IN OUT char **contentType :	Type of content
+*		OUT int *contentLength :	length of content
+*		OUT int *httpStatus :		HTTP status returned on receiving a
+*									response message
+*		IN int timeout :			time out value
+*
+*	Description :	Makes the HTTP GET message, connects to the peer, 
+*		sends the HTTP GET request, gets the response and parses the 
+*		response.
+*		If a proxy URL is defined then the connection is made there.
+*
+*	Return : int;
+*		UPNP_E_SUCCESS - On Success ;
+*		UPNP_E_INVALID_PARAM - Invalid Paramters ;
+*		UPNP_E_OUTOF_MEMORY ;
+*		UPNP_E_SOCKET_ERROR ;
+*		UPNP_E_BAD_RESPONSE ;
+*
+*	Note :
+*
+************************************************************************/
+int
+http_OpenHttpGetProxy( IN const char *url_str,
+                  IN const char *proxy_str,
+                  IN OUT void **Handle,
+                  IN OUT char **contentType,
+                  OUT int *contentLength,
+                  OUT int *httpStatus,
+                  IN int timeout )
 {
     int ret_code;
     int http_error_code;
@@ -1442,6 +1591,8 @@ http_OpenHttpGet( IN const char *url_str,
     membuffer request;
     http_get_handle_t *handle = NULL;
     uri_type url;
+    uri_type proxy;
+    uri_type *peer;
     parse_status_t status;
 
     if( ( !url_str ) || ( !Handle ) || ( !contentType )
@@ -1455,9 +1606,15 @@ http_OpenHttpGet( IN const char *url_str,
     ( *contentLength ) = 0;
 
     if( ( ret_code =
-          MakeGetMessage( url_str, &request, &url ) ) != UPNP_E_SUCCESS ) {
+          MakeGetMessage( url_str, proxy_str, &request, &url ) ) != UPNP_E_SUCCESS ) {
         return ret_code;
     }
+    if( proxy_str ) {
+        ret_code = http_FixStrUrl( ( char * )proxy_str, strlen( proxy_str ), &proxy );
+        peer = &proxy;
+    } else {
+        peer = &url;
+    }
 
     handle = ( http_get_handle_t * ) malloc( sizeof( http_get_handle_t ) );
 
@@ -1466,6 +1623,7 @@ http_OpenHttpGet( IN const char *url_str,
     }
 
     handle->entity_offset = 0;
+    handle->cancel = 0;
     parser_response_init( &handle->response, HTTPMETHOD_GET );
 
     tcp_connection = socket( AF_INET, SOCK_STREAM, 0 );
@@ -1482,7 +1640,7 @@ http_OpenHttpGet( IN const char *url_str,
     }
 
     ret_code = connect( handle->sock_info.socket,
-                        ( struct sockaddr * )&url.hostport.IPv4address,
+                        ( struct sockaddr * )&peer->hostport.IPv4address,
                         sizeof( struct sockaddr_in ) );
 
     if( ret_code == -1 ) {
@@ -1585,8 +1743,11 @@ http_SendStatusResponse( IN SOCKINFO * info,
     membuffer_init( &membuf );
     membuf.size_inc = 70;
 
-    ret = http_MakeMessage( &membuf, response_major, response_minor, "RSCB", http_status_code,  // response start line
-                            http_status_code ); // body
+    ret = http_MakeMessage(
+        &membuf, response_major, response_minor,
+        "RSCB",
+        http_status_code,  // response start line
+        http_status_code ); // body
     if( ret == 0 ) {
         timeout = HTTP_DEFAULT_TIMEOUT;
         ret = http_SendMessage( info, &timeout, "b",
@@ -1613,25 +1774,30 @@ http_SendStatusResponse( IN SOCKINFO * info,
 *		specified in the input parameters.
 *
 *		fmt types:
-*		's':	arg = const char* C_string
+*		'B':	arg = int status_code 
+*				appends content-length, content-type and HTML body for given code
 *		'b':	arg1 = const char* buf; arg2 = size_t buf_length 
 *				memory ptr
-*		'c':	(no args) appends CRLF "\r\n"
-*		'd':	arg = int number		// appends decimal number
-*		't':	arg = time_t * gmt_time	// appends time in RFC 1123 fmt
-*		'D':	(no args) appends HTTP DATE: header
-*		'S':	(no args) appends HTTP SERVER: header
-*		'U':	(no args) appends HTTP USER-AGENT: header
 *		'C':	(no args) appends a HTTP CONNECTION: close header 
 *				depending on major,minor version
+*		'c':	(no args) appends CRLF "\r\n"
+*		'D':	(no args) appends HTTP DATE: header
+*		'd':	arg = int number		// appends decimal number
+*		'G':	arg = range information         // add range header
+*		'h':	arg = off_t number		// appends off_t number
+*		'K':	(no args)                       // add chunky header
 *		'N':	arg1 = int content_length	// content-length header
+*               'q':    arg1 = http_method_t, arg2 = (uri_type *) // request start line and HOST header
 *		'Q':	arg1 = http_method_t; arg2 = char* url; 
 *				arg3 = int url_length // start line of request
 *		'R':	arg = int status_code // adds a response start line
-*		'B':	arg = int status_code 
-*				appends content-length, content-type and HTML body for given code
+*		'S':	(no args) appends HTTP SERVER: header
+*		's':	arg = const char* C_string
 *		'T':	arg = char * content_type; format e.g: "text/html";	
 *				 content-type header
+*		't':	arg = time_t * gmt_time	// appends time in RFC 1123 fmt
+*		'U':	(no args) appends HTTP USER-AGENT: header
+*               'X':    arg = const char useragent; "redsonic" HTTP X-User-Agent: useragent
 *
 *	Return : int;
 *		0 - On Success
@@ -1650,6 +1816,7 @@ http_MakeMessage( INOUT membuffer * buf,
     char c;
     char *s = NULL;
     int num;
+    off_t bignum;
     size_t length;
     time_t *loc_time;
     time_t curr_time;
@@ -1737,6 +1904,16 @@ http_MakeMessage( INOUT membuffer * buf,
             }
         }
 
+        else if( c == 'h' )     // off_t
+        {
+            bignum = ( off_t )va_arg( argp, off_t );
+
+            sprintf( tempbuf, "%lld", (int64_t)bignum );
+            if( membuffer_append( buf, tempbuf, strlen( tempbuf ) ) != 0 ) {
+                goto error_handler;
+            }
+        }
+
         else if( c == 't' || c == 'D' ) // date
         {
             if( c == 'D' ) {
@@ -1778,12 +1955,13 @@ http_MakeMessage( INOUT membuffer * buf,
 
         else if( c == 'N' ) {
             // content-length header
-            num = ( int )va_arg( argp, int );
+            bignum = ( off_t )va_arg( argp, off_t );
 
-            assert( num >= 0 );
-            if( http_MakeMessage
-                ( buf, http_major_version, http_minor_version, "sdc",
-                  "CONTENT-LENGTH: ", num ) != 0 ) {
+            assert( bignum >= 0 );
+            if (http_MakeMessage(
+                buf, http_major_version, http_minor_version,
+                "shc",
+                "CONTENT-LENGTH: ", bignum ) != 0 ) {
                 goto error_handler;
             }
         }
@@ -1793,13 +1971,28 @@ http_MakeMessage( INOUT membuffer * buf,
 
             temp_str = ( c == 'S' ) ? "SERVER: " : "USER-AGENT: ";
             get_sdk_info( tempbuf );
-            if( http_MakeMessage
-                ( buf, http_major_version, http_minor_version, "ss",
-                  temp_str, tempbuf ) != 0 ) {
+            if (http_MakeMessage(
+                buf, http_major_version, http_minor_version,
+                "ss",
+                temp_str, tempbuf ) != 0 ) {
                 goto error_handler;
             }
         }
 
+	else if( c == 'X' )          // C string
+        {
+            s = ( char * )va_arg( argp, char * );
+
+            assert( s );
+
+            if( membuffer_append_str( buf, "X-User-Agent: ") != 0 ) {
+                goto error_handler;
+            }
+            if( membuffer_append( buf, s, strlen( s ) ) != 0 ) {
+                goto error_handler;
+            }
+        }
+        
         else if( c == 'R' ) {
             // response start line
             //   e.g.: 'HTTP/1.1 200 OK'
@@ -1814,9 +2007,11 @@ http_MakeMessage( INOUT membuffer * buf,
 
             // str
             status_msg = http_get_code_text( status_code );
-            if( http_MakeMessage
-                ( buf, http_major_version, http_minor_version, "ssc",
-                  tempbuf, status_msg ) != 0 ) {
+            if (http_MakeMessage(
+                buf, http_major_version, http_minor_version,
+                "ssc",
+                tempbuf,
+                status_msg ) != 0 ) {
                 goto error_handler;
             }
         }
@@ -1831,11 +2026,14 @@ http_MakeMessage( INOUT membuffer * buf,
                      "<html><body><h1>",
                      status_code, http_get_code_text( status_code ),
                      "</h1></body></html>" );
-            num = strlen( tempbuf );
+            bignum = strlen( tempbuf );
 
-            if( http_MakeMessage( buf, http_major_version, http_minor_version, "NTcs", num, // content-length
-                                  "text/html",  // content-type
-                                  tempbuf ) != 0 )  // body
+            if (http_MakeMessage(
+                    buf, http_major_version, http_minor_version,
+                    "NTcs",
+                    bignum, // content-length
+                    "text/html",  // content-type
+                    tempbuf ) != 0 )  // body
             {
                 goto error_handler;
             }
@@ -1844,17 +2042,18 @@ http_MakeMessage( INOUT membuffer * buf,
         else if( c == 'Q' ) {
             // request start line
             // GET /foo/bar.html HTTP/1.1\r\n
-            //
 
             method = ( http_method_t ) va_arg( argp, http_method_t );
             method_str = method_to_str( method );
             url_str = ( const char * )va_arg( argp, const char * );
             num = ( int )va_arg( argp, int );   // length of url_str
 
-            if( http_MakeMessage( buf, http_major_version, http_minor_version, "ssbsdsdc", method_str,  // method
-                                  " ", url_str, num,    // url
-                                  " HTTP/", http_major_version, ".",
-                                  http_minor_version ) != 0 ) {
+            if (http_MakeMessage(
+                buf, http_major_version, http_minor_version,
+                "ssbsdsdc",
+                method_str,  // method
+                " ", url_str, num,    // url
+                " HTTP/", http_major_version, ".", http_minor_version ) != 0 ) {
                 goto error_handler;
             }
         }
@@ -1871,10 +2070,11 @@ http_MakeMessage( INOUT membuffer * buf,
                 goto error_handler;
             }
 
-            if( http_MakeMessage
-                ( buf, http_major_version, http_minor_version, "Q" "sbc",
-                  method, url.pathquery.buff, url.pathquery.size, "HOST: ",
-                  url.hostport.text.buff, url.hostport.text.size ) != 0 ) {
+            if (http_MakeMessage(
+                buf, http_major_version, http_minor_version,
+                "Q" "sbc",
+                method, url.pathquery.buff, url.pathquery.size,
+                "HOST: ", url.hostport.text.buff, url.hostport.text.size ) != 0 ) {
                 goto error_handler;
             }
         }
@@ -1883,9 +2083,10 @@ http_MakeMessage( INOUT membuffer * buf,
             // content type header
             temp_str = ( const char * )va_arg( argp, const char * );    // type/subtype format
 
-            if( http_MakeMessage
-                ( buf, http_major_version, http_minor_version, "ssc",
-                  "CONTENT-TYPE: ", temp_str ) != 0 ) {
+            if (http_MakeMessage(
+                buf, http_major_version, http_minor_version,
+		"ssc",
+                "CONTENT-TYPE: ", temp_str ) != 0 ) {
                 goto error_handler;
             }
         }
@@ -2009,16 +2210,13 @@ MakeGetMessageEx( const char *url_str,
                              hostlen );
              )
 
-            errCode = http_MakeMessage( request,
-                                        1,
-                                        1,
-                                        "QsbcGDCUc",
-                                        HTTPMETHOD_GET,
-                                        url->pathquery.buff,
-                                        url->pathquery.size,
-                                        "HOST: ",
-                                        hoststr,
-                                        hostlen, pRangeSpecifier );
+            errCode = http_MakeMessage(
+                request, 1, 1,
+                "QsbcGDCUc",
+                HTTPMETHOD_GET,
+                url->pathquery.buff, url->pathquery.size,
+                "HOST: ", hoststr, hostlen,
+                pRangeSpecifier );
 
         if( errCode != 0 ) {
             DBGONLY( UpnpPrintf( UPNP_INFO, HTTP, __FILE__, __LINE__,
@@ -2107,7 +2305,7 @@ http_OpenHttpGetEx( IN const char *url_str,
         }
 
         memset( &rangeBuf, 0, sizeof( rangeBuf ) );
-        sprintf( rangeBuf.RangeHeader, "Range: bytes=%d-%ld\r\n",
+        sprintf( rangeBuf.RangeHeader, "Range: bytes=%d-%d\r\n",
                  lowRange, highRange );
 
         membuffer_init( &request );
@@ -2226,9 +2424,18 @@ http_OpenHttpGetEx( IN const char *url_str,
 *	Note :
 ************************************************************************/
 // 'info' should have a size of at least 100 bytes
-XINLINE void
+void
 get_sdk_info( OUT char *info )
 {
+#ifdef WIN32
+ 	OSVERSIONINFO versioninfo;
+ 	versioninfo.dwOSVersionInfoSize=sizeof(OSVERSIONINFO);
+ 	if (GetVersionEx(&versioninfo)!=0)
+ 		sprintf( info, "%d.%d.%d %d/%s, UPnP/1.0, Portable SDK for UPnP devices/"PACKAGE_VERSION"\r\n",
+ 				versioninfo.dwMajorVersion, versioninfo.dwMinorVersion, versioninfo.dwBuildNumber, versioninfo.dwPlatformId, versioninfo.szCSDVersion );
+ 	else
+     *info = '\0';
+#else
     int ret_code;
     struct utsname sys_info;
 
@@ -2237,6 +2444,8 @@ get_sdk_info( OUT char *info )
         *info = '\0';
     }
 
-    sprintf( info, "%s/%s, UPnP/1.0, Intel SDK for UPnP devices /1.2\r\n",
+    sprintf( info, "%s/%s, UPnP/1.0, Portable SDK for UPnP devices/"
+	     PACKAGE_VERSION "\r\n",
              sys_info.sysname, sys_info.release );
+#endif
 }
