@@ -29,21 +29,27 @@
 //
 ///////////////////////////////////////////////////////////////////////////
 
+#include "config.h"
 #ifdef INCLUDE_DEVICE_APIS
 #if EXCLUDE_SOAP == 0
 
 #define SOAP_BODY "Body"
-#define SOAP_URN "http://schemas.xmlsoap.org/soap/envelope/"
+#define SOAP_URN "http:/""/schemas.xmlsoap.org/soap/envelope/"
 
 #define QUERY_STATE_VAR_URN "urn:schemas-upnp-org:control-1-0"
 
-#include "config.h"
 #include "upnpapi.h"
 #include "parsetools.h"
 #include "statcodes.h"
 #include "httpparser.h"
 #include "httpreadwrite.h"
 #include "unixutil.h"
+#include "soaplib.h"
+#include "ssdplib.h"
+
+#ifdef WIN32
+ #define snprintf _snprintf
+#endif
 
 // timeout duration in secs for transmission/reception
 #define SOAP_TIMEOUT UPNP_TIMEOUT
@@ -183,13 +189,14 @@ send_error_response( IN SOCKINFO * info,
                      IN const char *err_msg,
                      IN http_message_t * hmsg )
 {
-    int content_length;
+    off_t content_length;
     int timeout_secs = SOAP_TIMEOUT;
     int major,
       minor;
     const char *start_body =
-        "<s:Envelope\n"
-        "xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\"\n"
+//		"<?xml version=\"1.0\"?>\n" required??
+        "<s:Envelope "
+        "xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" "
         "s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">\n"
         "<s:Body>\n"
         "<s:Fault>\n"
@@ -221,14 +228,16 @@ send_error_response( IN SOCKINFO * info,
 
     // make headers
     membuffer_init( &headers );
-    if( http_MakeMessage( &headers, major, minor,
-                          "RNsDsSc" "sssss",
-                          500,
-                          content_length,
-                          ContentTypeHeader,
-                          "EXT:\r\n",
-                          start_body, err_code_str, mid_body, err_msg,
-                          end_body ) != 0 ) {
+    if (http_MakeMessage(
+        &headers, major, minor,
+        "RNsDsSXcc" "sssss",
+        500,
+        content_length,
+        ContentTypeHeader,
+        "EXT:\r\n",
+        X_USER_AGENT,
+        start_body, err_code_str, mid_body, err_msg,
+        end_body ) != 0 ) {
         membuffer_destroy( &headers );
         return;                 // out of mem
     }
@@ -258,13 +267,13 @@ send_var_query_response( IN SOCKINFO * info,
                          IN const char *var_value,
                          IN http_message_t * hmsg )
 {
-    int content_length;
+    off_t content_length;
     int timeout_secs = SOAP_TIMEOUT;
-    int major,
-      minor;
+    int major;
+    int minor;
     const char *start_body =
-        "<s:Envelope\n"
-        "xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\"\n"
+        "<s:Envelope "
+        "xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" "
         "s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">\n"
         "<s:Body>\n"
         "<u:QueryStateVariableResponse "
@@ -285,16 +294,20 @@ send_var_query_response( IN SOCKINFO * info,
 
     // make headers
     membuffer_init( &response );
-    if( http_MakeMessage( &response, major, minor,
-                          "RNsDsSc" "sss",
-                          HTTP_OK,
-                          content_length,
-                          ContentTypeHeader,
-                          "EXT:\r\n",
-                          start_body, var_value, end_body ) != 0 ) {
+    
+    if (http_MakeMessage(
+        &response, major, minor,
+        "RNsDsSXcc" "sss",
+        HTTP_OK,
+        content_length,
+        ContentTypeHeader,
+        "EXT:\r\n",
+        X_USER_AGENT,
+        start_body, var_value, end_body ) != 0 ) {
         membuffer_destroy( &response );
         return;                 // out of mem
     }
+    
     // send msg
     http_SendMessage( info, &timeout_secs, "b",
                       response.buf, response.length );
@@ -369,7 +382,7 @@ get_action_node( IN IXML_Document * TempDoc,
     if( strstr( nodeName, NodeName ) == NULL ) {
         goto error_handler;
     } else {
-        ActNodeName = ixmlPrintDocument( ActNode );
+        ActNodeName = ixmlPrintNode( ActNode );
         if( ActNodeName == NULL ) {
             goto error_handler;
         }
@@ -587,13 +600,13 @@ get_device_info( IN http_message_t * request,
     service_info *serv_info;
     char save_char;
     int ret_code = -1;          // error by default
-    char *control_url;
+    const char *control_url;
     char *actionName = NULL;
 
     // null-terminate pathquery of url
     control_url = request->uri.pathquery.buff;
     save_char = control_url[request->uri.pathquery.size];
-    control_url[request->uri.pathquery.size] = '\0';
+    ((char *)control_url)[request->uri.pathquery.size] = '\0';
 
     HandleLock(  );
 
@@ -650,7 +663,7 @@ get_device_info( IN http_message_t * request,
     ret_code = 0;
 
   error_handler:
-    control_url[request->uri.pathquery.size] = save_char;   // restore
+    ((char *)control_url)[request->uri.pathquery.size] = save_char;   // restore
     HandleUnlock(  );
     return ret_code;
 }
@@ -679,10 +692,11 @@ send_action_response( IN SOCKINFO * info,
     int major,
       minor;
     int err_code;
-    int content_length;
+    off_t content_length;
     int ret_code;
     int timeout_secs = SOAP_TIMEOUT;
     static char *start_body =
+//        "<?xml version=\"1.0\"?>" required??
         "<s:Envelope xmlns:s=\"http://schemas.xmlsoap."
         "org/soap/envelope/\" s:encodingStyle=\"http://schemas.xmlsoap."
         "org/soap/encoding/\"><s:Body>\n";
@@ -695,20 +709,28 @@ send_action_response( IN SOCKINFO * info,
     err_code = UPNP_E_OUTOF_MEMORY; // one error only
 
     // get xml
-    xml_response = ixmlPrintDocument( ( IXML_Node * ) action_resp );
+    xml_response = ixmlPrintNode( ( IXML_Node * ) action_resp );
     if( xml_response == NULL ) {
         goto error_handler;
     }
 
-    content_length = strlen( start_body ) + strlen( xml_response ) +
+    content_length =
+        strlen( start_body ) +
+        strlen( xml_response ) +
         strlen( end_body );
 
     // make headers
-    if( http_MakeMessage( &headers, major, minor, "RNsDsSc", HTTP_OK,   // status code
-                          content_length, ContentTypeHeader, "EXT:\r\n" // EXT header
-         ) != 0 ) {
+    if (http_MakeMessage(
+        &headers, major, minor,
+        "RNsDsSXcc",
+        HTTP_OK,   // status code
+        content_length,
+        ContentTypeHeader,
+        "EXT:\r\n",
+        X_USER_AGENT) != 0 ) {
         goto error_handler;
     }
+
     // send whole msg
     ret_code = http_SendMessage( info, &timeout_secs, "bbbb",
                                  headers.buf, headers.length,
@@ -758,7 +780,7 @@ get_var_name( IN IXML_Document * TempDoc,
     IXML_Node *VarNameNode = NULL;
     IXML_Node *VarNode = NULL;
     const DOMString StNodeName = NULL;
-    DOMString Temp = NULL;
+    const DOMString Temp = NULL;
     int ret_val = -1;
 
     // Got the Envelop node here
