@@ -42,6 +42,12 @@
 #include "httpparser.h"
 #include "httpreadwrite.h"
 
+#ifdef WIN32
+ #include <winsock2.h>
+ #include <ws2tcpip.h>
+ #include "unixutil.h"
+#endif
+
 #define MAX_TIME_TOREAD  45
 
 CLIENTONLY( SOCKET gSsdpReqSocket = 0;
@@ -106,7 +112,7 @@ CLIENTONLY( SOCKET gSsdpReqSocket = 0;
     IXML_Node *tmpNode = NULL;
     IXML_Node *tmpNode2 = NULL;
     IXML_Node *textNode = NULL;
-    DOMString tmpStr;
+    const DOMString tmpStr;
     char SERVER[200];
 
     DBGONLY( const DOMString dbgStr;
@@ -455,14 +461,17 @@ CLIENTONLY( SOCKET gSsdpReqSocket = 0;
 int
 Make_Socket_NoBlocking( int sock )
 {
-
+#ifdef WIN32
+     u_long val=1;
+     return ioctlsocket(sock, FIONBIO, &val);
+#else
     int val;
 
     val = fcntl( sock, F_GETFL, 0 );
     if( fcntl( sock, F_SETFL, val | O_NONBLOCK ) == -1 ) {
         return -1;
     }
-
+#endif
     return 0;
 }
 
@@ -809,7 +818,7 @@ readFromSSDPSocket( SOCKET socket )
     struct sockaddr_in clientAddr;
     ThreadPoolJob job;
     ssdp_thread_data *data = NULL;
-    int socklen = 0;
+    socklen_t socklen = 0;
     int byteReceived = 0;
 
     requestBuf = staticBuf;
@@ -913,6 +922,7 @@ get_ssdp_sockets( MiniServerSockArray * out )
     struct ip_mreq ssdpMcastAddr;
     struct sockaddr_in ssdpAddr;
     int option = 1;
+    struct in_addr addr;
 
     CLIENTONLY( if( ( ssdpReqSock = socket( AF_INET, SOCK_DGRAM, 0 ) )
                     == UPNP_INVALID_SOCKET ) {
@@ -951,6 +961,22 @@ get_ssdp_sockets( MiniServerSockArray * out )
         UpnpCloseSocket( ssdpSock );
         return UPNP_E_SOCKET_ERROR;
     }
+    
+    #ifdef __FreeBSD__
+    if( setsockopt( ssdpSock, SOL_SOCKET, SO_REUSEPORT,
+                    ( char * )&onOff, sizeof( onOff ) ) != 0 ) {
+
+        DBGONLY( UpnpPrintf( UPNP_CRITICAL,
+                             SSDP, __FILE__, __LINE__,
+                             "Error in set reuse port !!!\n" );
+             )
+            CLIENTONLY( shutdown( ssdpReqSock, SD_BOTH ) );
+        CLIENTONLY( UpnpCloseSocket( ssdpReqSock ) );
+        shutdown( ssdpSock, SD_BOTH );
+        UpnpCloseSocket( ssdpSock );
+        return UPNP_E_SOCKET_ERROR;
+    }
+    #endif
 
     memset( ( void * )&ssdpAddr, 0, sizeof( struct sockaddr_in ) );
     ssdpAddr.sin_family = AF_INET;
@@ -972,7 +998,7 @@ get_ssdp_sockets( MiniServerSockArray * out )
     }
 
     memset( ( void * )&ssdpMcastAddr, 0, sizeof( struct ip_mreq ) );
-    ssdpMcastAddr.imr_interface.s_addr = htonl( INADDR_ANY );
+    ssdpMcastAddr.imr_interface.s_addr = inet_addr( LOCAL_HOST );
     ssdpMcastAddr.imr_multiaddr.s_addr = inet_addr( SSDP_IP );
     if( setsockopt( ssdpSock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
                     ( char * )&ssdpMcastAddr,
@@ -987,6 +1013,17 @@ get_ssdp_sockets( MiniServerSockArray * out )
         CLIENTONLY( UpnpCloseSocket( ssdpReqSock ) );
         return UPNP_E_SOCKET_ERROR;
     }
+
+    /* Set multicast interface. */
+    memset( ( void * )&addr, 0, sizeof( struct in_addr ));
+    addr.s_addr = inet_addr(LOCAL_HOST);
+    if (setsockopt(ssdpSock, IPPROTO_IP, IP_MULTICAST_IF,
+                   (char *)&addr, sizeof addr) != 0) {
+        DBGONLY(UpnpPrintf( UPNP_INFO, SSDP, __FILE__, __LINE__,
+                            "Couldn't set multicast interface.\n" ));
+        /* This is probably not a critical error, so let's continue. */
+    }
+
     // result is not checked becuase it will fail in WinMe and Win9x.
     setsockopt( ssdpSock, IPPROTO_IP,
                 IP_MULTICAST_TTL, &ttl, sizeof( ttl ) );
